<?php
if (!class_exists('wpdreams_searchTerms')) {
    class wpdreams_searchTerms extends wpdreams_search {

        private $parts = array();
        private $remaining_limit;
        private $query;

        protected function do_search() {
            global $wpdb;
            global $q_config;

            $options = $this->options;
            $searchData = $this->searchData;

            $parts = array();
            $relevance_parts = array();
            $types = array();

            $kw_logic = w_isset_def($searchData['keyword_logic'], 'or');
            $q_config['language'] = $options['qtranslate_lang'];

            $s = $this->s; // full keyword
            $_s = $this->_s;    // array of keywords

            if ($kw_logic == 'orex')
                $_si = "[[:<:]]" . implode('[[:>:]]|[[:<:]]', $_s) . "[[:>:]]"; // imploded exact phrase for regexp
            else
                $_si = implode('|', $_s);                                       // imploded phrase for regexp

            $_si = $_si!=''?$_si:$s;

            $q_config['language'] = $options['qtranslate_lang'];
            $words = $options['set_exactonly']==1 ? array($s) : $_s;
            $regexp_words = count($_s > 0) ? implode('|', $_s) : $s;


            /*----------------------- Gather Types --------------------------*/
            if ($searchData['return_categories'] == 1)
                $types[] = "category";
            if (isset($searchData['selected-return_terms']) && is_array($searchData['selected-return_terms']) && count($searchData['selected-return_terms']) > 0)
                $types = array_merge($types, $searchData['selected-return_terms']);
            if (count($types) < 1) {
                return '';
            } else {
                $twords = implode('[[:>:]]|[[:<:]]', $types);
                $taxonomies = "($wpdb->term_taxonomy.taxonomy REGEXP '[[:<:]]".$twords."[[:>:]]')";
            }
            /*---------------------------------------------------------------*/


            /*----------------------- Title query ---------------------------*/

            if ($kw_logic == 'or' || $kw_logic == 'and') {
                $op = strtoupper($kw_logic);
                if (count($_s)>0)
                    $_like = implode("%' ".$op." lower($wpdb->terms.name) LIKE '%", $words);
                else
                    $_like = $s;
                $parts[] = "( lower($wpdb->terms.name) LIKE '%".$_like."%' )";
            } else {
                $_like = array();
                $op = $kw_logic == 'andex' ? 'AND' : 'OR';
                foreach ($words as $word) {
                    $_like[] = "
                           (lower($wpdb->terms.name) LIKE '% ".$word." %'
                        OR  lower($wpdb->terms.name) LIKE '".$word." %'
                        OR  lower($wpdb->terms.name) LIKE '% ".$word."'
                        OR  lower($wpdb->terms.name) = '".$word."')";
                }
                $parts[] = "(" . implode(' '.$op.' ', $_like) . ")";
            }

            /*$words = $options['set_exactonly']==1?$s:$_si;
            if ($kw_logic == 'or' || $kw_logic == 'orex') {
                $parts[] = "(lower($wpdb->terms.name) REGEXP '$words')";
            } else {
                if (count($_s)>0)
                    $and_like = implode("$r_sign' AND lower($wpdb->terms.name) RLIKE '$l_sign", $_s);
                else
                    $and_like = $s;
                $parts[] = "lower($wpdb->terms.name) RLIKE '$l_sign".$and_like."$r_sign'";
            }*/
            $relevance_parts[] = "(case when
            (lower($wpdb->terms.name) REGEXP '$regexp_words')
             then $searchData[titleweight] else 0 end)";
            $relevance_parts[] = "(case when
            (lower($wpdb->terms.name) REGEXP '$s')
             then $searchData[etitleweight] else 0 end)";

            // The first word relevance is higher
            if (count($_s)>0)
                $relevance_parts[] = "(case when
              (lower($wpdb->terms.name) REGEXP '".$_s[0]."')
               then $searchData[etitleweight] else 0 end)";

            /*---------------------------------------------------------------*/



            /*------------------------- Build like --------------------------*/
            $like_query = implode(' OR ', $parts);
            if ($like_query == "")
                $like_query = "(1)";
            else {
                $like_query = "($like_query)";
            }
            /*---------------------------------------------------------------*/

            /*---------------------- Build relevance ------------------------*/
            $relevance = implode(' + ', $relevance_parts);
            if ($searchData['userelevance'] != 1 || $relevance == "")
                $relevance = "(1)";
            else {
                $relevance = "($relevance)";
            }
            /*---------------------------------------------------------------*/


            $querystr = "
    		SELECT 
          $wpdb->terms.name as title,
          $wpdb->terms.term_id as id,
          '' as content,
          '' as date,
          '' as author,
          $wpdb->term_taxonomy.taxonomy as taxonomy,
          $relevance as relevance
    		FROM $wpdb->terms
        LEFT JOIN $wpdb->term_taxonomy ON $wpdb->terms.term_id = $wpdb->term_taxonomy.term_id
    	WHERE
    	    $taxonomies
            AND $like_query
        GROUP BY
          $wpdb->terms.term_id
         ";
            $querystr .= " ORDER BY relevance DESC, $wpdb->terms.name ASC
        LIMIT " . $searchData['maxresults'];

            $term_res = $wpdb->get_results($querystr, OBJECT);
            //var_dump($querystr);die("!!");
            //var_dump($term_res);die("!!");

            $this->results = $term_res;

            return $term_res;

        }

        protected function post_process() {

            $term_res = is_array($this->results)?$this->results:array();
            $options = $this->options;
            $searchData = $this->searchData;
            $s = $this->s;
            $_s = $this->_s;

            foreach ($term_res as $k=>$v) {
                $term_res[$k]->link = get_term_link( (int)$v->id, $v->taxonomy);
            }

            return $term_res;

        }

    }
}
?>